﻿Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    Friend VideoInDevice As String = ""


    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function

    ' =======================================================================================================
    '   Files
    ' =======================================================================================================
    ' returns lower-case extension with initial dot
    Public Function GetExtension(ByVal str As String) As String
        On Error Resume Next
        Return LCase(IO.Path.GetExtension(str))
    End Function
    Public Function RemoveExtension(ByVal str As String) As String
        On Error Resume Next
        Return PlatformAdjustedFileName(IO.Path.GetDirectoryName(str) & "\" & IO.Path.GetFileNameWithoutExtension(str))
    End Function
    Public Function FileExists(ByVal fileName As String) As Boolean
        Return My.Computer.FileSystem.FileExists(fileName)
    End Function
    Public Function FolderExists(ByVal FolderName As String) As Boolean
        If FolderName.Length < 2 Then Return False
        FolderName = LCase(FolderName)
        Select Case FolderName
            Case "a:\", "b:\", "c:\", "d:\", "e:\", "f:\", "g:\", "h:\", "i:\", "j:\", "k:\"
                Return True
        End Select
        Return My.Computer.FileSystem.DirectoryExists(FolderName)
    End Function


    ' =======================================================================================================
    '   Platform selection
    ' =======================================================================================================
    Friend Function CanUseWindowsDlls() As Boolean
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
                Return True
            Case Else
                Return False
        End Select
    End Function
    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                             Optional ByVal DefaultName As String = "") As String
        If CanUseWindowsDlls() Then
            FileName = Replace(FileName, "/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = Replace(FileName, "\", "/")
            Return FileName
        End If
    End Function


    ' =======================================================================================
    '  FORM
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private sw As Integer
    Private sh As Integer
    Private Sub GetMaxScreenBounds()
        ' -------------------------------------- Gets an array of all the screens connected to the system.
        Dim Screens() As System.Windows.Forms.Screen = System.Windows.Forms.Screen.AllScreens
        ' -------------------------------------- For each screen
        Dim Index As Integer
        For Index = 0 To Screens.GetUpperBound(0)
            sw = Math.Max(sw, Screens(Index).Bounds.Left + Screens(Index).Bounds.Width)
            sh = Math.Max(sh, Screens(Index).Bounds.Top + Screens(Index).Bounds.Height)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, sw)           ' non oltre le dimensioni di una maximized window
    '        .Height = Math.Min(.Height, sh)         ' non oltre le dimensioni di una maximized window
    '        .Width = Math.Max(.Width, 32)           ' non meno di 32x24
    '        .Height = Math.Max(.Height, 24)         ' non meno di 32x24
    '        .Left = Math.Min(.Left, sw - .Width)    ' non oltre il bordo destro
    '        .Top = Math.Min(.Top, sh - .Height)     ' non oltre il bordo inferiore
    '        .Left = Math.Max(.Left, 0)              ' non meno del bordo sinistro
    '        .Top = Math.Max(.Top, 0)                ' non sopra al bordo superiore
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, sw)               ' non oltre le dimensioni del VIRTUALSCREEN
            .Height = Math.Min(.Height, sh)
            .Width = Math.Max(.Width, 32)               ' non meno di 32x24
            .Height = Math.Max(.Height, 24)             ' non meno di 32x24
            .Left = Math.Min(.Left, sw - 50)            ' non oltre il bordo destro - 50
            .Top = Math.Min(.Top, sh - 50)              ' non oltre il bordo inferiore - 50
            .Left = Math.Max(.Left, 50 - .Width)        ' non meno del bordo sinistro + 50
            .Top = Math.Max(.Top, -30)                  ' non sopra al bordo superiore
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function



    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function

    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================

    Friend Sub Save_INI()
        '
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        Dim f As System.IO.StreamWriter
        f = IO.File.CreateText(iniFileName)
        '
        f.WriteLine(" Program Params")
        f.WriteLine("===========================================")
        Dim r As Rectangle
        r = GetFormRectangle(Form_Main)
        f.WriteLine(TabString("FormMain_Top", r.Top))
        f.WriteLine(TabString("FormMain_Left", r.Left))
        'f.WriteLine(TabString("FormMain_Width", r.Width))
        'f.WriteLine(TabString("FormMain_Height", r.Height))
        f.WriteLine(TabString("FormMain_WindowState", Form_Main.WindowState))
        '
        f.WriteLine("")
        f.WriteLine(" Video Input Params")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("VideoInDevice", VideoInDevice))
        f.WriteLine(TabString("VideoFormat", VideoFormatParams.VideoFormat))
        f.WriteLine(TabString("VideoSize", VideoFormatParams.VideoSize))
        f.WriteLine(TabString("VideoFPS", VideoFormatParams.VideoFPS))
        '
        f.WriteLine("")
        f.WriteLine(" Crop Params")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("Zoom", Form_Main.txt_Zoom.NumericValue.ToString))
        f.WriteLine(TabString("ShiftX", Form_Main.txt_ShiftX.NumericValue.ToString))
        f.WriteLine(TabString("ShiftY", Form_Main.txt_ShiftY.NumericValue.ToString))
        f.WriteLine(TabString("FlipX", Form_Main.CheckBox_FlipX.Checked.ToString))
        f.WriteLine(TabString("FlipY", Form_Main.CheckBox_FlipY.Checked.ToString))
        '
        f.WriteLine("")
        f.WriteLine(" Motion detector")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("Positive", Form_Main.RadioButton_Positive.Checked.ToString))
        f.WriteLine(TabString("Negative", Form_Main.RadioButton_Negative.Checked.ToString))
        f.WriteLine(TabString("Double", Form_Main.RadioButton_Double.Checked.ToString))
        f.WriteLine(TabString("GrayScale", Form_Main.CheckBox_GrayScale.Checked.ToString))
        f.WriteLine(TabString("BlackOrWhite", Form_Main.CheckBox_BlackOrWhite.Checked.ToString))
        '
        f.WriteLine(TabString("DetectorShift", Form_Main.txt_DetectorShift.NumericValue.ToString))
        f.WriteLine(TabString("DetectorGain", Form_Main.txt_DetectorGain.NumericValue.ToString))
        f.WriteLine(TabString("DetectorRes", Form_Main.txt_DetectorRes.NumericValue.ToString))
        f.WriteLine(TabString("DelayStages", Form_Main.txt_DelayStages.NumericValueInteger.ToString))
        f.WriteLine(TabString("DetectorMaxFps", Form_Main.txt_DetectorMaxFps.NumericValueInteger.ToString))
        '
        f.WriteLine("")
        f.WriteLine(" Process quality")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("ZoomQuality", Form_Main.txt_ZoomQuality.NumericValueInteger.ToString))
        f.WriteLine(TabString("PreFilterQuality", Form_Main.txt_PreFilterQuality.NumericValueInteger.ToString))
        f.WriteLine(TabString("PostFilterQuality", Form_Main.txt_PostFilterQuality.NumericValueInteger.ToString))
        '
        f.WriteLine("")
        f.WriteLine(" Enable")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("EnableImage1", Form_Main.CheckBox_EnableImage1.Checked.ToString))
        f.WriteLine(TabString("EnableImage2", Form_Main.CheckBox_EnableImage2.Checked.ToString))
        f.WriteLine(TabString("TrigShow1", Form_Main.CheckBox_TrigShow1.Checked.ToString))
        f.WriteLine(TabString("TrigShow2", Form_Main.CheckBox_TrigShow2.Checked.ToString))
        '
        f.WriteLine("")
        f.WriteLine(" AreaParams")
        f.WriteLine("===========================================")
        f.WriteLine(TabString("UsedAreas", Form_Main.txt_UsedAreas.NumericValueInteger.ToString))
        f.WriteLine(TabString("FirstSlot", Form_Main.txt_FirstSlot.NumericValueInteger.ToString))
        '
        f.WriteLine("")
        f.WriteLine(" Areas")
        f.WriteLine("===========================================")
        TriggerAreas_WriteAreasToFile(f)
        f.Close()
    End Sub


    Friend Sub Load_INI()
        ' ------------------------------------------------------------- default values
        VideoInDevice = ""
        VideoFormatParams.VideoFormat = "RGB42"
        VideoFormatParams.VideoSize = "320 x 240"
        VideoFormatParams.VideoFPS = "30"
        ' ------------------------------------------------------------- 
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"

        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)
            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "FormMain_Top" : Form_Main.Top = CInt(Val(l))
                    Case "FormMain_Left" : Form_Main.Left = CInt(Val(l))
                        'Case "FormMain_Width" : Form_Main.Width = CInt(Val(l))
                        'Case "FormMain_Height" : Form_Main.Height = CInt(Val(l))
                    Case "FormMain_WindowState" : Form_Main.WindowState = CType((Val(l)), FormWindowState)
                        ' ------------------------------------------------------------------------------ Video in device
                    Case "VideoInDevice" : VideoInDevice = l
                    Case "VideoFormat" : VideoFormatParams.VideoFormat = l
                    Case "VideoSize" : VideoFormatParams.VideoSize = l
                    Case "VideoFPS" : VideoFormatParams.VideoFPS = l
                        ' ------------------------------------------------------------------------------ Image crop
                    Case "Zoom" : Form_Main.txt_Zoom.NumericValue = Val_Double(l)
                    Case "ShiftX" : Form_Main.txt_ShiftX.NumericValue = Val_Double(l)
                    Case "ShiftY" : Form_Main.txt_ShiftY.NumericValue = Val_Double(l)
                    Case "FlipX" : Form_Main.CheckBox_FlipX.Checked = l = "True"
                    Case "FlipY" : Form_Main.CheckBox_FlipY.Checked = l = "True"
                        ' ------------------------------------------------------------------------------ Motion detector 
                    Case "Positive" : Form_Main.RadioButton_Positive.Checked = l = "True"
                    Case "Negative" : Form_Main.RadioButton_Negative.Checked = l = "True"
                    Case "Double" : Form_Main.RadioButton_Double.Checked = l = "True"
                    Case "GrayScale" : Form_Main.CheckBox_GrayScale.Checked = l = "True"
                    Case "BlackOrWhite" : Form_Main.CheckBox_BlackOrWhite.Checked = l = "True"
                        '
                    Case "DetectorShift" : Form_Main.txt_DetectorShift.NumericValue = Val_Double(l)
                    Case "DetectorGain" : Form_Main.txt_DetectorGain.NumericValue = Val_Double(l)
                    Case "DetectorRes" : Form_Main.txt_DetectorRes.NumericValue = Val_Double(l)
                    Case "DelayStages" : Form_Main.txt_DelayStages.NumericValueInteger = Val_Int(l)
                    Case "DetectorMaxFps" : Form_Main.txt_DetectorMaxFps.NumericValueInteger = Val_Int(l)
                        ' ------------------------------------------------------------------------------ Process quality
                    Case "ZoomQuality" : Form_Main.txt_ZoomQuality.NumericValueInteger = Val_Int(l)
                    Case "PreFilterQuality" : Form_Main.txt_PreFilterQuality.NumericValueInteger = Val_Int(l)
                    Case "PostFilterQuality" : Form_Main.txt_PostFilterQuality.NumericValueInteger = Val_Int(l)
                        ' ------------------------------------------------------------------------------ Enable
                    Case "EnableImage1" : Form_Main.CheckBox_EnableImage1.Checked = l = "True"
                    Case "EnableImage2" : Form_Main.CheckBox_EnableImage2.Checked = l = "True"
                    Case "TrigShow1" : Form_Main.CheckBox_TrigShow1.Checked = l = "True"
                    Case "TrigShow2" : Form_Main.CheckBox_TrigShow2.Checked = l = "True"
                        ' ------------------------------------------------------------------------------ Area params
                    Case "UsedAreas" : Form_Main.txt_UsedAreas.NumericValueInteger = Val_Int(l)
                    Case "FirstSlot" : Form_Main.txt_FirstSlot.NumericValueInteger = Val_Int(l)
                        ' ------------------------------------------------------------------------------ Areas
                        TriggerAreas_ReadAreasFromFile(f)
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form_Main)
    End Sub


End Module
